=== modified file 'configure.ac'
--- configure.ac	2008-11-13 22:41:09 +0000
+++ configure.ac	2008-11-16 02:29:43 +0000
@@ -347,7 +347,6 @@
 fi
 
 # We use libtool
-#AC_LIBTOOL_WIN32_DLL
 AC_PROG_LIBTOOL
 
 # Ensure that we have --preserve-dup-deps defines, otherwise we get link
@@ -1566,7 +1565,9 @@
 AC_SUBST(MAKE_BINARY_DISTRIBUTION_OPTIONS)
 
 dnl GCC Precompiled Header Support
-AM_CONDITIONAL([GCC_PCH],[test "$GCC" = "yes"])
+dnl re-enable later
+dnl AM_CONDITIONAL([GCC_PCH],[test "$GCC" = "yes"])
+AM_CONDITIONAL([GCC_PCH],[test "no" = "yes"])
 
 AC_CONFIG_FILES(Makefile extra/Makefile mysys/Makefile dnl
  mystrings/Makefile storage/Makefile dnl

=== modified file 'drizzled/Makefile.am'
--- drizzled/Makefile.am	2008-11-13 00:26:12 +0000
+++ drizzled/Makefile.am	2008-11-15 18:39:51 +0000
@@ -98,6 +98,7 @@
 		data_home.h \
 		db.h \
 		definitions.h \
+		dtcollation.h \
 		errmsg.h \
 		error_injection.h \
 		field.h \
@@ -109,6 +110,10 @@
 		ha_statistics.h \
 		handler.h \
 		handler_structs.h \
+		hybrid_type.h \
+		hybrid_type_traits.h \
+		hybrid_type_traits_decimal.h \
+		hybrid_type_traits_integer.h \
 		item_cmpfunc.h \
 		item_create.h \
 		item_func.h \
@@ -183,12 +188,16 @@
 		db.cc \
 		discover.cc \
 		drizzled.cc \
+		dtcollation.cc \
 		errmsg.cc \
 		error.cc \
 		field.cc \
 		field_conv.cc \
 		field_iterator.cc \
 		filesort.cc \
+		hybrid_type_traits.cc \
+		hybrid_type_traits_decimal.cc \
+		hybrid_type_traits_integer.cc \
 		init.cc \
 		item_buff.cc \
 		item.cc \

=== modified file 'drizzled/definitions.h'
--- drizzled/definitions.h	2008-11-13 20:13:17 +0000
+++ drizzled/definitions.h	2008-11-15 18:39:51 +0000
@@ -971,6 +971,57 @@
 #define OPEN_FRM_ONLY   1                // open FRM file only
 #define OPEN_FULL_TABLE 2                // open FRM,MYD, MYI files
 
+/*
+   "Declared Type Collation"
+   A combination of collation and its derivation.
+
+  Flags for collation aggregation modes:
+  MY_COLL_ALLOW_SUPERSET_CONV  - allow conversion to a superset
+  MY_COLL_ALLOW_COERCIBLE_CONV - allow conversion of a coercible value
+                                 (i.e. constant).
+  MY_COLL_ALLOW_CONV           - allow any kind of conversion
+                                 (combination of the above two)
+  MY_COLL_DISALLOW_NONE        - don't allow return DERIVATION_NONE
+                                 (e.g. when aggregating for comparison)
+  MY_COLL_CMP_CONV             - combination of MY_COLL_ALLOW_CONV
+                                 and MY_COLL_DISALLOW_NONE
+*/
+
+#define MY_COLL_ALLOW_SUPERSET_CONV   1
+#define MY_COLL_ALLOW_COERCIBLE_CONV  2
+#define MY_COLL_ALLOW_CONV            3
+#define MY_COLL_DISALLOW_NONE         4
+#define MY_COLL_CMP_CONV              7
+
+
+/*
+  This enum is used to report information about monotonicity of function
+  represented by Item* tree.
+  Monotonicity is defined only for Item* trees that represent table
+  partitioning expressions (i.e. have no subselects/user vars/PS parameters
+  etc etc). An Item* tree is assumed to have the same monotonicity properties
+  as its correspoinding function F:
+
+  [signed] int64_t F(field1, field2, ...) {
+    put values of field_i into table record buffer;
+    return item->val_int(); 
+  }
+
+  NOTE
+  At the moment function monotonicity is not well defined (and so may be
+  incorrect) for Item trees with parameters/return types that are different
+  from INT_RESULT, may be NULL, or are unsigned.
+  It will be possible to address this issue once the related partitioning bugs
+  (BUG#16002, BUG#15447, BUG#13436) are fixed.
+*/
+
+typedef enum monotonicity_info 
+{
+   NON_MONOTONIC,              /* none of the below holds */
+   MONOTONIC_INCREASING,       /* F() is unary and (x < y) => (F(x) <= F(y)) */
+   MONOTONIC_STRICT_INCREASING /* F() is unary and (x < y) => (F(x) <  F(y)) */
+} enum_monotonicity_info;
+
 
 #endif /* DRIZZLE_SERVER_DEFINITIONS_H */
 

=== added file 'drizzled/dtcollation.cc'
--- drizzled/dtcollation.cc	1970-01-01 00:00:00 +0000
+++ drizzled/dtcollation.cc	2008-11-14 23:27:02 +0000
@@ -0,0 +1,373 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2008 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; version 2 of the License.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#include <drizzled/server_includes.h>
+#include <drizzled/definitions.h>
+#include <mysys/my_sys.h>
+#include <mystrings/m_ctype.h>
+#include <drizzled/error.h>
+
+#include <drizzled/dtcollation.h>
+
+
+static bool
+left_is_superset(const DTCollation &left, const DTCollation &right)
+{
+  /* Allow convert to Unicode */
+  if (left.collation->state & MY_CS_UNICODE &&
+      (left.derivation < right.derivation ||
+       (left.derivation == right.derivation &&
+        !(right.collation->state & MY_CS_UNICODE))))
+    return true;
+  /* Allow convert from ASCII */
+  if (right.repertoire == MY_REPERTOIRE_ASCII &&
+      (left.derivation < right.derivation ||
+       (left.derivation == right.derivation &&
+        !(left.repertoire == MY_REPERTOIRE_ASCII))))
+    return true;
+  /* Disallow conversion otherwise */
+  return false;
+}
+
+
+DTCollation::DTCollation()
+{
+  collation= &my_charset_bin;
+  derivation= DERIVATION_NONE;
+  repertoire= MY_REPERTOIRE_UNICODE30;
+}
+
+
+DTCollation::DTCollation(const CHARSET_INFO * const collation_arg,
+                         Derivation derivation_arg)
+{
+  collation= collation_arg;
+  derivation= derivation_arg;
+  set_repertoire_from_charset(collation_arg);
+}
+void DTCollation::set_repertoire_from_charset(const CHARSET_INFO * const cs)
+{
+  repertoire= cs->state & MY_CS_PUREASCII ?
+    MY_REPERTOIRE_ASCII : MY_REPERTOIRE_UNICODE30;
+}
+
+
+void DTCollation::set(DTCollation &dt)
+{
+  collation= dt.collation;
+  derivation= dt.derivation;
+  repertoire= dt.repertoire;
+}
+
+
+void DTCollation::set(const CHARSET_INFO * const collation_arg,
+                      Derivation derivation_arg)
+{
+  collation= collation_arg;
+  derivation= derivation_arg;
+  set_repertoire_from_charset(collation_arg);
+}
+
+
+void DTCollation::set(const CHARSET_INFO * const collation_arg,
+                      Derivation derivation_arg,
+                      uint32_t repertoire_arg)
+{
+  collation= collation_arg;
+  derivation= derivation_arg;
+  repertoire= repertoire_arg;
+}
+
+
+void DTCollation::set(const CHARSET_INFO * const collation_arg)
+{
+  collation= collation_arg;
+  set_repertoire_from_charset(collation_arg);
+}
+
+
+void DTCollation::set(Derivation derivation_arg)
+{
+  derivation= derivation_arg;
+}
+
+
+bool DTCollation::aggregate(DTCollation &dt, uint32_t flags)
+{
+  if (!my_charset_same(collation, dt.collation))
+  {
+    /*
+      We do allow to use binary strings (like BLOBS)
+      together with character strings.
+      Binaries have more precedence than a character
+      string of the same derivation.
+    */
+    if (collation == &my_charset_bin)
+    {
+      if (derivation <= dt.derivation)
+        ; // Do nothing
+      else
+      {
+        set(dt);
+      }
+    }
+    else if (dt.collation == &my_charset_bin)
+    {
+      if (dt.derivation <= derivation)
+      {
+        set(dt);
+      }
+      else
+      {
+        // Do nothing
+      }
+    }
+    else if ((flags & MY_COLL_ALLOW_SUPERSET_CONV) &&
+             left_is_superset(*this, dt))
+    {
+      // Do nothing
+    }
+    else if ((flags & MY_COLL_ALLOW_SUPERSET_CONV) &&
+             left_is_superset(dt, *this))
+    {
+      set(dt);
+    }
+    else if ((flags & MY_COLL_ALLOW_COERCIBLE_CONV) &&
+             derivation < dt.derivation &&
+             dt.derivation >= DERIVATION_SYSCONST)
+    {
+      // Do nothing;
+    }
+    else if ((flags & MY_COLL_ALLOW_COERCIBLE_CONV) &&
+             dt.derivation < derivation &&
+             derivation >= DERIVATION_SYSCONST)
+    {
+      set(dt);
+    }
+    else
+    {
+      // Cannot apply conversion
+      set(0, DERIVATION_NONE, 0);
+      return 1;
+    }
+  }
+  else if (derivation < dt.derivation)
+  {
+    // Do nothing
+  }
+  else if (dt.derivation < derivation)
+  {
+    set(dt);
+  }
+  else
+  {
+    if (collation == dt.collation)
+    {
+      // Do nothing
+    }
+    else
+    {
+      if (derivation == DERIVATION_EXPLICIT)
+      {
+        set(0, DERIVATION_NONE, 0);
+        return 1;
+      }
+      if (collation->state & MY_CS_BINSORT)
+        return 0;
+      if (dt.collation->state & MY_CS_BINSORT)
+      {
+        set(dt);
+        return 0;
+      }
+      const CHARSET_INFO * const bin= get_charset_by_csname(collation->csname,
+                                                            MY_CS_BINSORT,
+                                                            MYF(0));
+      set(bin, DERIVATION_NONE);
+    }
+  }
+  repertoire|= dt.repertoire;
+  return 0;
+}
+
+
+bool DTCollation::set(DTCollation &dt1, DTCollation &dt2, uint32_t flags)
+{ set(dt1); return aggregate(dt2, flags); }
+
+
+const char *DTCollation::derivation_name() const
+{
+  switch(derivation)
+  {
+  case DERIVATION_IGNORABLE: return "IGNORABLE";
+  case DERIVATION_COERCIBLE: return "COERCIBLE";
+  case DERIVATION_IMPLICIT:  return "IMPLICIT";
+  case DERIVATION_SYSCONST:  return "SYSCONST";
+  case DERIVATION_EXPLICIT:  return "EXPLICIT";
+  case DERIVATION_NONE:      return "NONE";
+  default: return "UNKNOWN";
+  }
+}
+
+
+bool agg_item_collations(DTCollation &c, const char *fname,
+                         Item **av, uint32_t count,
+                         uint32_t flags, int item_sep)
+{
+  uint32_t i;
+  Item **arg;
+  c.set(av[0]->collation);
+  for (i= 1, arg= &av[item_sep]; i < count; i++, arg++)
+  {
+    if (c.aggregate((*arg)->collation, flags))
+    {
+      my_coll_agg_error(av, count, fname, item_sep);
+      return true;
+    }
+  }
+  if ((flags & MY_COLL_DISALLOW_NONE) &&
+      c.derivation == DERIVATION_NONE)
+  {
+    my_coll_agg_error(av, count, fname, item_sep);
+    return true;
+  }
+  return false;
+}
+
+
+bool agg_item_collations_for_comparison(DTCollation &c, const char *fname,
+                                        Item **av, uint32_t count,
+                                        uint32_t flags)
+{
+  return (agg_item_collations(c, fname, av, count,
+                              flags | MY_COLL_DISALLOW_NONE, 1));
+}
+
+
+bool agg_item_charsets(DTCollation &coll, const char *fname,
+                       Item **args, uint32_t nargs, uint32_t flags,
+                       int item_sep)
+{
+  Item **arg, *safe_args[2];
+
+  memset(safe_args, 0, sizeof(safe_args));
+
+  if (agg_item_collations(coll, fname, args, nargs, flags, item_sep))
+    return true;
+
+  /*
+    For better error reporting: save the first and the second argument.
+    We need this only if the the number of args is 3 or 2:
+    - for a longer argument list, "Illegal mix of collations"
+      doesn't display each argument's characteristics.
+    - if nargs is 1, then this error cannot happen.
+  */
+  if (nargs >=2 && nargs <= 3)
+  {
+    safe_args[0]= args[0];
+    safe_args[1]= args[item_sep];
+  }
+
+  Session *session= current_session;
+  bool res= false;
+  uint32_t i;
+
+  for (i= 0, arg= args; i < nargs; i++, arg+= item_sep)
+  {
+    Item* conv;
+    uint32_t dummy_offset;
+    if (!String::needs_conversion(0, (*arg)->collation.collation,
+                                  coll.collation,
+                                  &dummy_offset))
+      continue;
+
+    if (!(conv= (*arg)->safe_charset_converter(coll.collation)) &&
+        ((*arg)->collation.repertoire == MY_REPERTOIRE_ASCII))
+      conv= new Item_func_conv_charset(*arg, coll.collation, 1);
+
+    if (!conv)
+    {
+      if (nargs >=2 && nargs <= 3)
+      {
+        /* restore the original arguments for better error message */
+        args[0]= safe_args[0];
+        args[item_sep]= safe_args[1];
+      }
+      my_coll_agg_error(args, nargs, fname, item_sep);
+      res= true;
+      break; // we cannot return here, we need to restore "arena".
+    }
+    if ((*arg)->type() == Item::FIELD_ITEM)
+      ((Item_field *)(*arg))->no_const_subst= 1;
+    /*
+      If in statement prepare, then we create a converter for two
+      constant items, do it once and then reuse it.
+      If we're in execution of a prepared statement, arena is NULL,
+      and the conv was created in runtime memory. This can be
+      the case only if the argument is a parameter marker ('?'),
+      because for all true constants the charset converter has already
+      been created in prepare. In this case register the change for
+      rollback.
+    */
+    session->change_item_tree(arg, conv);
+    /*
+      We do not check conv->fixed, because Item_func_conv_charset which can
+      be return by safe_charset_converter can't be fixed at creation
+    */
+    conv->fix_fields(session, arg);
+  }
+
+  return res;
+}
+
+
+void my_coll_agg_error(DTCollation &c1,
+                       DTCollation &c2, const char *fname)
+{
+  my_error(ER_CANT_AGGREGATE_2COLLATIONS,MYF(0),
+           c1.collation->name,c1.derivation_name(),
+           c2.collation->name,c2.derivation_name(),
+           fname);
+}
+
+
+void my_coll_agg_error(DTCollation &c1,
+                       DTCollation &c2,
+                       DTCollation &c3,
+                       const char *fname)
+{
+  my_error(ER_CANT_AGGREGATE_3COLLATIONS,MYF(0),
+           c1.collation->name,c1.derivation_name(),
+           c2.collation->name,c2.derivation_name(),
+           c3.collation->name,c3.derivation_name(),
+           fname);
+}
+
+
+void my_coll_agg_error(Item** args, uint32_t count, const char *fname,
+                       int item_sep)
+{
+  if (count == 2)
+    my_coll_agg_error(args[0]->collation, args[item_sep]->collation, fname);
+  else if (count == 3)
+    my_coll_agg_error(args[0]->collation, args[item_sep]->collation,
+                      args[2*item_sep]->collation, fname);
+  else
+    my_error(ER_CANT_AGGREGATE_NCOLLATIONS,MYF(0),fname);
+}
+

=== added file 'drizzled/dtcollation.h'
--- drizzled/dtcollation.h	1970-01-01 00:00:00 +0000
+++ drizzled/dtcollation.h	2008-11-14 23:27:02 +0000
@@ -0,0 +1,143 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2008 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; version 2 of the License.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef DRIZZLED_DTCOLLATION_H
+#define DRIZZLED_DTCOLLATION_H
+
+#include <stdint.h>
+
+#include <drizzled/definitions.h>
+
+class Item;
+typedef struct charset_info_st CHARSET_INFO;
+
+
+class DTCollation {
+public:
+  const CHARSET_INFO *collation;
+  enum Derivation derivation;
+  uint32_t repertoire;
+
+  void set_repertoire_from_charset(const CHARSET_INFO * const cs);
+
+  DTCollation();
+  DTCollation(const CHARSET_INFO * const collation_arg,
+              Derivation derivation_arg);
+  void set(DTCollation &dt);
+  void set(const CHARSET_INFO * const collation_arg,
+           Derivation derivation_arg);
+  void set(const CHARSET_INFO * const collation_arg,
+           Derivation derivation_arg,
+           uint32_t repertoire_arg);
+  void set(const CHARSET_INFO * const collation_arg);
+  void set(Derivation derivation_arg);
+  bool set(DTCollation &dt1, DTCollation &dt2, uint32_t flags= 0);
+
+/**
+  Aggregate two collations together taking
+  into account their coercibility (aka derivation):.
+
+  0 == DERIVATION_EXPLICIT  - an explicitly written COLLATE clause @n
+  1 == DERIVATION_NONE      - a mix of two different collations @n
+  2 == DERIVATION_IMPLICIT  - a column @n
+  3 == DERIVATION_COERCIBLE - a string constant.
+
+  The most important rules are:
+  -# If collations are the same:
+  chose this collation, and the strongest derivation.
+  -# If collations are different:
+  - Character sets may differ, but only if conversion without
+  data loss is possible. The caller provides flags whether
+  character set conversion attempts should be done. If no
+  flags are substituted, then the character sets must be the same.
+  Currently processed flags are:
+  MY_COLL_ALLOW_SUPERSET_CONV  - allow conversion to a superset
+  MY_COLL_ALLOW_COERCIBLE_CONV - allow conversion of a coercible value
+  - two EXPLICIT collations produce an error, e.g. this is wrong:
+  CONCAT(expr1 collate latin1_swedish_ci, expr2 collate latin1_german_ci)
+  - the side with smaller derivation value wins,
+  i.e. a column is stronger than a string constant,
+  an explicit COLLATE clause is stronger than a column.
+  - if derivations are the same, we have DERIVATION_NONE,
+  we'll wait for an explicit COLLATE clause which possibly can
+  come from another argument later: for example, this is valid,
+  but we don't know yet when collecting the first two arguments:
+     @code
+       CONCAT(latin1_swedish_ci_column,
+              latin1_german1_ci_column,
+              expr COLLATE latin1_german2_ci)
+  @endcode
+*/
+
+  bool aggregate(DTCollation &dt, uint32_t flags= 0);
+
+  const char *derivation_name() const;
+
+};
+
+
+bool agg_item_collations(DTCollation &c, const char *name,
+                         Item **items, uint32_t nitems,
+                         uint32_t flags, int item_sep);
+bool agg_item_collations_for_comparison(DTCollation &c, const char *name,
+                                        Item **items, uint32_t nitems,
+                                        uint32_t flags);
+
+/**
+  Collect arguments' character sets together.
+
+  We allow to apply automatic character set conversion in some cases.
+  The conditions when conversion is possible are:
+  - arguments A and B have different charsets
+  - A wins according to coercibility rules
+    (i.e. a column is stronger than a string constant,
+     an explicit COLLATE clause is stronger than a column)
+  - character set of A is either superset for character set of B,
+    or B is a string constant which can be converted into the
+    character set of A without data loss.
+
+  If all of the above is true, then it's possible to convert
+  B into the character set of A, and then compare according
+  to the collation of A.
+
+  For functions with more than two arguments:
+  @code
+    collect(A,B,C) ::= collect(collect(A,B),C)
+  @endcode
+  Since this function calls Session::change_item_tree() on the passed Item **
+  pointers, it is necessary to pass the original Item **'s, not copies.
+  Otherwise their values will not be properly restored (see BUG#20769).
+  If the items are not consecutive (eg. args[2] and args[5]), use the
+  item_sep argument, ie.
+  @code
+    agg_item_charsets(coll, fname, &args[2], 2, flags, 3)
+  @endcode
+*/
+bool agg_item_charsets(DTCollation &c, const char *name,
+                       Item **items, uint32_t nitems, uint32_t flags,
+                       int item_sep);
+
+
+void my_coll_agg_error(DTCollation &c1, DTCollation &c2, const char *fname);
+void my_coll_agg_error(DTCollation &c1, DTCollation &c2, DTCollation &c3,
+                       const char *fname);
+void my_coll_agg_error(Item** args, uint32_t count, const char *fname,
+                       int item_sep);
+
+#endif /* DRIZZLED_DTCOLLATION_H */

=== added file 'drizzled/hybrid_type.h'
--- drizzled/hybrid_type.h	1970-01-01 00:00:00 +0000
+++ drizzled/hybrid_type.h	2008-11-15 18:39:51 +0000
@@ -0,0 +1,64 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2008 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; version 2 of the License.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef DRIZZLED_HYBRID_TYPE_H
+#define DRIZZLED_HYBRID_TYPE_H
+
+#include <stdint.h>
+#include <drizzled/my_decimal.h>
+
+/*************************************************************************/
+/*
+  A framework to easily handle different return types for hybrid items
+  (hybrid item is an item whose operand can be of any type, e.g. integer,
+  real, decimal).
+*/
+
+class Hybrid_type_traits;
+
+class Hybrid_type
+{
+public:
+  int64_t integer;
+
+  double real;
+  /*
+    Use two decimal buffers interchangeably to speed up += operation
+    which has no native support in decimal library.
+    Hybrid_type+= arg is implemented as dec_buf[1]= dec_buf[0] + arg.
+    The third decimal is used as a handy temporary storage.
+  */
+  my_decimal dec_buf[3];
+  int used_dec_buf_no;
+
+  /*
+    Traits moved to a separate class to
+      a) be able to easily change object traits in runtime
+      b) they work as a differentiator for the union above
+  */
+  const Hybrid_type_traits *traits;
+
+  Hybrid_type() {}
+  /* XXX: add traits->copy() when needed */
+  Hybrid_type(const Hybrid_type &rhs) :traits(rhs.traits) {}
+};
+
+
+
+#endif /* DRIZZLED_HYBRID_TYPE_H */

=== added file 'drizzled/hybrid_type_traits.cc'
--- drizzled/hybrid_type_traits.cc	1970-01-01 00:00:00 +0000
+++ drizzled/hybrid_type_traits.cc	2008-11-15 18:39:51 +0000
@@ -0,0 +1,102 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2008 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; version 2 of the License.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#include <drizzled/server_includes.h>
+#include <drizzled/hybrid_type_traits.h>
+#include CMATH_H
+
+#if defined(CMATH_NAMESPACE)
+using namespace CMATH_NAMESPACE;
+#endif
+
+static const Hybrid_type_traits real_traits_instance;
+
+Item_result Hybrid_type_traits::type() const
+{
+  return REAL_RESULT;
+}
+
+
+void Hybrid_type_traits::fix_length_and_dec(Item *item, Item *arg) const
+{
+  item->decimals= NOT_FIXED_DEC;
+  item->max_length= item->float_length(arg->decimals);
+}
+
+
+/* Hybrid_type operations. */
+void Hybrid_type_traits::set_zero(Hybrid_type *val) const
+{
+  val->real= 0.0;
+}
+
+
+void Hybrid_type_traits::add(Hybrid_type *val, Field *f) const
+{
+  val->real+= f->val_real();
+}
+
+
+void Hybrid_type_traits::div(Hybrid_type *val, uint64_t u) const
+{
+  val->real/= uint64_t2double(u);
+}
+
+
+int64_t Hybrid_type_traits::val_int(Hybrid_type *val,
+                                    bool) const
+{
+  return (int64_t) rint(val->real);
+}
+
+
+double Hybrid_type_traits::val_real(Hybrid_type *val) const
+{
+  return val->real;
+}
+
+
+my_decimal *
+Hybrid_type_traits::val_decimal(Hybrid_type *val, my_decimal *) const
+{
+  double2my_decimal(E_DEC_FATAL_ERROR, val->real, val->dec_buf);
+  return val->dec_buf;
+}
+
+
+String *
+Hybrid_type_traits::val_str(Hybrid_type *val, String *to,
+                            uint8_t decimals) const
+{
+  to->set_real(val->real, decimals, &my_charset_bin);
+  return to;
+}
+
+
+const Hybrid_type_traits *Hybrid_type_traits::instance()
+{
+  return &real_traits_instance;
+}
+
+
+Hybrid_type_traits::Hybrid_type_traits()
+{}
+
+Hybrid_type_traits::~Hybrid_type_traits()
+{}

=== added file 'drizzled/hybrid_type_traits.h'
--- drizzled/hybrid_type_traits.h	1970-01-01 00:00:00 +0000
+++ drizzled/hybrid_type_traits.h	2008-11-15 18:39:51 +0000
@@ -0,0 +1,60 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2008 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; version 2 of the License.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef DRIZZLED_HYBRID_TYPE_TRAITS_H
+#define DRIZZLED_HYBRID_TYPE_TRAITS_H
+
+#include <stdint.h>
+
+class Item;
+class Field;
+class Hybrid_type;
+class String;
+class my_decimal;
+
+/* Hybryd_type_traits interface + default implementation for REAL_RESULT */
+
+class Hybrid_type_traits
+{
+public:
+
+  virtual Item_result type() const;
+
+  virtual void
+  fix_length_and_dec(Item *item, Item *arg) const;
+
+  /* Hybrid_type operations. */
+  virtual void set_zero(Hybrid_type *val) const;
+  virtual void add(Hybrid_type *val, Field *f) const;
+  virtual void div(Hybrid_type *val, uint64_t u) const;
+
+  virtual int64_t val_int(Hybrid_type *val,
+                          bool unsigned_flag) const;
+  virtual double val_real(Hybrid_type *val) const;
+  virtual my_decimal *val_decimal(Hybrid_type *val, my_decimal *buf) const;
+  virtual String *val_str(Hybrid_type *val, String *buf,
+                          uint8_t decimals) const;
+
+  static const Hybrid_type_traits *instance();
+  Hybrid_type_traits();
+  virtual ~Hybrid_type_traits();
+
+};
+
+#endif /* DRIZZLED_HYBRID_TYPE_TRAITS_H */

=== added file 'drizzled/hybrid_type_traits_decimal.cc'
--- drizzled/hybrid_type_traits_decimal.cc	1970-01-01 00:00:00 +0000
+++ drizzled/hybrid_type_traits_decimal.cc	2008-11-15 18:39:51 +0000
@@ -0,0 +1,116 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2008 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; version 2 of the License.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#include <drizzled/server_includes.h>
+#include <drizzled/definitions.h>
+
+#include <drizzled/hybrid_type_traits_decimal.h>
+
+/* Hybrid_type_traits_decimal */
+static const Hybrid_type_traits_decimal decimal_traits_instance;
+
+
+Item_result Hybrid_type_traits_decimal::type() const { return DECIMAL_RESULT; }
+
+
+void
+Hybrid_type_traits_decimal::fix_length_and_dec(Item *item, Item *arg) const
+{
+  item->decimals= arg->decimals;
+  item->max_length= cmin(arg->max_length + DECIMAL_LONGLONG_DIGITS,
+                        (unsigned int)DECIMAL_MAX_STR_LENGTH);
+}
+
+
+void Hybrid_type_traits_decimal::set_zero(Hybrid_type *val) const
+{
+  my_decimal_set_zero(&val->dec_buf[0]);
+  val->used_dec_buf_no= 0;
+}
+
+
+void Hybrid_type_traits_decimal::add(Hybrid_type *val, Field *f) const
+{
+  my_decimal_add(E_DEC_FATAL_ERROR,
+                 &val->dec_buf[val->used_dec_buf_no ^ 1],
+                 &val->dec_buf[val->used_dec_buf_no],
+                 f->val_decimal(&val->dec_buf[2]));
+  val->used_dec_buf_no^= 1;
+}
+
+
+/**
+  @todo
+  what is '4' for scale?
+*/
+void Hybrid_type_traits_decimal::div(Hybrid_type *val, uint64_t u) const
+{
+  int2my_decimal(E_DEC_FATAL_ERROR, u, true, &val->dec_buf[2]);
+  /* XXX: what is '4' for scale? */
+  my_decimal_div(E_DEC_FATAL_ERROR,
+                 &val->dec_buf[val->used_dec_buf_no ^ 1],
+                 &val->dec_buf[val->used_dec_buf_no],
+                 &val->dec_buf[2], 4);
+  val->used_dec_buf_no^= 1;
+}
+
+
+int64_t
+Hybrid_type_traits_decimal::val_int(Hybrid_type *val, bool unsigned_flag) const
+{
+  int64_t result;
+  my_decimal2int(E_DEC_FATAL_ERROR, &val->dec_buf[val->used_dec_buf_no],
+                 unsigned_flag, &result);
+  return result;
+}
+
+
+double
+Hybrid_type_traits_decimal::val_real(Hybrid_type *val) const
+{
+  my_decimal2double(E_DEC_FATAL_ERROR, &val->dec_buf[val->used_dec_buf_no],
+                    &val->real);
+  return val->real;
+}
+
+
+my_decimal *Hybrid_type_traits_decimal::val_decimal(Hybrid_type *val,
+                                                    my_decimal *) const
+{ return &val->dec_buf[val->used_dec_buf_no]; }
+
+
+String *
+Hybrid_type_traits_decimal::val_str(Hybrid_type *val, String *to,
+                                    uint8_t decimals) const
+{
+  my_decimal_round(E_DEC_FATAL_ERROR, &val->dec_buf[val->used_dec_buf_no],
+                   decimals, false, &val->dec_buf[2]);
+  my_decimal2string(E_DEC_FATAL_ERROR, &val->dec_buf[2], 0, 0, 0, to);
+  return to;
+}
+
+
+const Hybrid_type_traits_decimal *Hybrid_type_traits_decimal::instance()
+{
+  return &decimal_traits_instance;
+}
+
+
+Hybrid_type_traits_decimal::Hybrid_type_traits_decimal()
+{}

=== added file 'drizzled/hybrid_type_traits_decimal.h'
--- drizzled/hybrid_type_traits_decimal.h	1970-01-01 00:00:00 +0000
+++ drizzled/hybrid_type_traits_decimal.h	2008-11-15 18:39:51 +0000
@@ -0,0 +1,56 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2008 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; version 2 of the License.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef DRIZZLED_HYBRID_TYPE_TRAITS_DECIMAL_H
+#define DRIZZLED_HYBRID_TYPE_TRAITS_DECIMAL_H
+
+#include <stdint.h>
+#include <drizzled/hybrid_type_traits.h>
+
+class Item;
+class Field;
+class Hybrid_type;
+class my_decimal;
+class String;
+
+class Hybrid_type_traits_decimal: public Hybrid_type_traits
+{
+public:
+  virtual Item_result type() const;
+
+  virtual void
+  fix_length_and_dec(Item *arg, Item *item) const;
+
+  /* Hybrid_type operations. */
+  virtual void set_zero(Hybrid_type *val) const;
+  virtual void add(Hybrid_type *val, Field *f) const;
+  virtual void div(Hybrid_type *val, uint64_t u) const;
+
+  virtual int64_t val_int(Hybrid_type *val, bool unsigned_flag) const;
+  virtual double val_real(Hybrid_type *val) const;
+  virtual my_decimal *val_decimal(Hybrid_type *val,
+                                  my_decimal *buf) const;
+  virtual String *val_str(Hybrid_type *val, String *buf,
+                          uint8_t decimals) const;
+  static const Hybrid_type_traits_decimal *instance();
+  Hybrid_type_traits_decimal();
+};
+
+
+#endif /* DRIZZLED_HYBRID_TYPE_TRAITS_DECIMAL_H */

=== added file 'drizzled/hybrid_type_traits_integer.cc'
--- drizzled/hybrid_type_traits_integer.cc	1970-01-01 00:00:00 +0000
+++ drizzled/hybrid_type_traits_integer.cc	2008-11-15 18:39:51 +0000
@@ -0,0 +1,102 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2008 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; version 2 of the License.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+
+#include <drizzled/server_includes.h>
+#include <drizzled/definitions.h>
+
+#include <drizzled/hybrid_type_traits_integer.h>
+
+
+/* Hybrid_type_traits_integer */
+static const Hybrid_type_traits_integer integer_traits_instance;
+
+
+Item_result Hybrid_type_traits_integer::type() const
+{
+  return INT_RESULT;
+}
+
+
+void
+Hybrid_type_traits_integer::fix_length_and_dec(Item *item, Item *) const
+{
+  item->decimals= 0;
+  item->max_length= MY_INT64_NUM_DECIMAL_DIGITS;
+  item->unsigned_flag= 0;
+}
+
+
+/* Hybrid_type operations. */
+void Hybrid_type_traits_integer::set_zero(Hybrid_type *val) const
+{
+  val->integer= 0;
+}
+
+
+void Hybrid_type_traits_integer::add(Hybrid_type *val, Field *f) const
+{
+  val->integer+= f->val_int();
+}
+
+
+void Hybrid_type_traits_integer::div(Hybrid_type *val, uint64_t u) const
+{
+  val->integer/= (int64_t) u;
+}
+
+
+int64_t Hybrid_type_traits_integer::val_int(Hybrid_type *val, bool) const
+{
+  return val->integer;
+}
+
+
+double Hybrid_type_traits_integer::val_real(Hybrid_type *val) const
+{
+  return (double) val->integer;
+}
+
+
+my_decimal *Hybrid_type_traits_integer::val_decimal(Hybrid_type *val,
+                                                    my_decimal *) const
+{
+  int2my_decimal(E_DEC_FATAL_ERROR, val->integer, 0, &val->dec_buf[2]);
+  return &val->dec_buf[2];
+}
+
+
+String *Hybrid_type_traits_integer::val_str(Hybrid_type *val, String *buf,
+                                            uint8_t) const
+{
+  buf->set(val->integer, &my_charset_bin);
+  return buf;
+}
+
+
+const Hybrid_type_traits_integer *Hybrid_type_traits_integer::instance()
+{
+  return &integer_traits_instance;
+}
+
+
+Hybrid_type_traits_integer::Hybrid_type_traits_integer()
+{}
+
+

=== added file 'drizzled/hybrid_type_traits_integer.h'
--- drizzled/hybrid_type_traits_integer.h	1970-01-01 00:00:00 +0000
+++ drizzled/hybrid_type_traits_integer.h	2008-11-15 18:39:51 +0000
@@ -0,0 +1,55 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2008 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; version 2 of the License.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef DRIZZLED_HYBRID_TYPE_TRAITS_INTEGER_H
+#define DRIZZLED_HYBRID_TYPE_TRAITS_INTEGER_H
+
+#include <stdint.h>
+#include <drizzled/hybrid_type_traits.h>
+
+class Item;
+class Field;
+class Hybrid_type;
+class my_decimal;
+class String;
+
+class Hybrid_type_traits_integer: public Hybrid_type_traits
+{
+
+public:
+  virtual Item_result type() const;
+
+  virtual void fix_length_and_dec(Item *arg, Item *item) const;
+
+  /* Hybrid_type operations. */
+  virtual void set_zero(Hybrid_type *val) const;
+  virtual void add(Hybrid_type *val, Field *f) const;
+  virtual void div(Hybrid_type *val, uint64_t u) const;
+
+  virtual int64_t val_int(Hybrid_type *val, bool unsigned_flag) const;
+  virtual double val_real(Hybrid_type *val) const;
+  virtual my_decimal *val_decimal(Hybrid_type *val, my_decimal *buf) const;
+  virtual String *val_str(Hybrid_type *val, String *buf,
+                          uint8_t decimals) const;
+
+  static const Hybrid_type_traits_integer *instance();
+  Hybrid_type_traits_integer();
+};
+
+#endif /* HYBRID_TYPE_TRAITS_INTEGER_H */

=== modified file 'drizzled/item.cc'
--- drizzled/item.cc	2008-11-15 17:58:57 +0000
+++ drizzled/item.cc	2008-11-16 02:30:08 +0000
@@ -27,139 +27,6 @@
 
 const String my_null_string("NULL", 4, default_charset_info);
 
-/****************************************************************************/
-
-/* Hybrid_type_traits {_real} */
-
-void Hybrid_type_traits::fix_length_and_dec(Item *item, Item *arg) const
-{
-  item->decimals= NOT_FIXED_DEC;
-  item->max_length= item->float_length(arg->decimals);
-}
-
-static const Hybrid_type_traits real_traits_instance;
-
-const Hybrid_type_traits *Hybrid_type_traits::instance()
-{
-  return &real_traits_instance;
-}
-
-int64_t Hybrid_type_traits::val_int(Hybrid_type *val,
-                                    bool) const
-{
-  return (int64_t) rint(val->real);
-}
-
-my_decimal *
-Hybrid_type_traits::val_decimal(Hybrid_type *val, my_decimal *) const
-{
-  double2my_decimal(E_DEC_FATAL_ERROR, val->real, val->dec_buf);
-  return val->dec_buf;
-}
-
-
-String *
-Hybrid_type_traits::val_str(Hybrid_type *val, String *to, uint8_t decimals) const
-{
-  to->set_real(val->real, decimals, &my_charset_bin);
-  return to;
-}
-
-/* Hybrid_type_traits_decimal */
-static const Hybrid_type_traits_decimal decimal_traits_instance;
-
-const Hybrid_type_traits_decimal *Hybrid_type_traits_decimal::instance()
-{
-  return &decimal_traits_instance;
-}
-
-
-void
-Hybrid_type_traits_decimal::fix_length_and_dec(Item *item, Item *arg) const
-{
-  item->decimals= arg->decimals;
-  item->max_length= cmin(arg->max_length + DECIMAL_LONGLONG_DIGITS,
-                        (unsigned int)DECIMAL_MAX_STR_LENGTH);
-}
-
-
-void Hybrid_type_traits_decimal::set_zero(Hybrid_type *val) const
-{
-  my_decimal_set_zero(&val->dec_buf[0]);
-  val->used_dec_buf_no= 0;
-}
-
-
-void Hybrid_type_traits_decimal::add(Hybrid_type *val, Field *f) const
-{
-  my_decimal_add(E_DEC_FATAL_ERROR,
-                 &val->dec_buf[val->used_dec_buf_no ^ 1],
-                 &val->dec_buf[val->used_dec_buf_no],
-                 f->val_decimal(&val->dec_buf[2]));
-  val->used_dec_buf_no^= 1;
-}
-
-
-/**
-  @todo
-  what is '4' for scale?
-*/
-void Hybrid_type_traits_decimal::div(Hybrid_type *val, uint64_t u) const
-{
-  int2my_decimal(E_DEC_FATAL_ERROR, u, true, &val->dec_buf[2]);
-  /* XXX: what is '4' for scale? */
-  my_decimal_div(E_DEC_FATAL_ERROR,
-                 &val->dec_buf[val->used_dec_buf_no ^ 1],
-                 &val->dec_buf[val->used_dec_buf_no],
-                 &val->dec_buf[2], 4);
-  val->used_dec_buf_no^= 1;
-}
-
-
-int64_t
-Hybrid_type_traits_decimal::val_int(Hybrid_type *val, bool unsigned_flag) const
-{
-  int64_t result;
-  my_decimal2int(E_DEC_FATAL_ERROR, &val->dec_buf[val->used_dec_buf_no],
-                 unsigned_flag, &result);
-  return result;
-}
-
-
-double
-Hybrid_type_traits_decimal::val_real(Hybrid_type *val) const
-{
-  my_decimal2double(E_DEC_FATAL_ERROR, &val->dec_buf[val->used_dec_buf_no],
-                    &val->real);
-  return val->real;
-}
-
-
-String *
-Hybrid_type_traits_decimal::val_str(Hybrid_type *val, String *to,
-                                    uint8_t decimals) const
-{
-  my_decimal_round(E_DEC_FATAL_ERROR, &val->dec_buf[val->used_dec_buf_no],
-                   decimals, false, &val->dec_buf[2]);
-  my_decimal2string(E_DEC_FATAL_ERROR, &val->dec_buf[2], 0, 0, 0, to);
-  return to;
-}
-
-/* Hybrid_type_traits_integer */
-static const Hybrid_type_traits_integer integer_traits_instance;
-
-const Hybrid_type_traits_integer *Hybrid_type_traits_integer::instance()
-{
-  return &integer_traits_instance;
-}
-
-void
-Hybrid_type_traits_integer::fix_length_and_dec(Item *item, Item *) const
-{
-  item->decimals= 0;
-  item->max_length= MY_INT64_NUM_DECIMAL_DIGITS;
-  item->unsigned_flag= 0;
-}
 
 /*****************************************************************************
 ** Item functions
@@ -1080,333 +947,6 @@
 }
 
 
-static bool
-left_is_superset(DTCollation *left, DTCollation *right)
-{
-  /* Allow convert to Unicode */
-  if (left->collation->state & MY_CS_UNICODE &&
-      (left->derivation < right->derivation ||
-       (left->derivation == right->derivation &&
-        !(right->collation->state & MY_CS_UNICODE))))
-    return true;
-  /* Allow convert from ASCII */
-  if (right->repertoire == MY_REPERTOIRE_ASCII &&
-      (left->derivation < right->derivation ||
-       (left->derivation == right->derivation &&
-        !(left->repertoire == MY_REPERTOIRE_ASCII))))
-    return true;
-  /* Disallow conversion otherwise */
-  return false;
-}
-
-/**
-  Aggregate two collations together taking
-  into account their coercibility (aka derivation):.
-
-  0 == DERIVATION_EXPLICIT  - an explicitly written COLLATE clause @n
-  1 == DERIVATION_NONE      - a mix of two different collations @n
-  2 == DERIVATION_IMPLICIT  - a column @n
-  3 == DERIVATION_COERCIBLE - a string constant.
-
-  The most important rules are:
-  -# If collations are the same:
-  chose this collation, and the strongest derivation.
-  -# If collations are different:
-  - Character sets may differ, but only if conversion without
-  data loss is possible. The caller provides flags whether
-  character set conversion attempts should be done. If no
-  flags are substituted, then the character sets must be the same.
-  Currently processed flags are:
-  MY_COLL_ALLOW_SUPERSET_CONV  - allow conversion to a superset
-  MY_COLL_ALLOW_COERCIBLE_CONV - allow conversion of a coercible value
-  - two EXPLICIT collations produce an error, e.g. this is wrong:
-  CONCAT(expr1 collate latin1_swedish_ci, expr2 collate latin1_german_ci)
-  - the side with smaller derivation value wins,
-  i.e. a column is stronger than a string constant,
-  an explicit COLLATE clause is stronger than a column.
-  - if derivations are the same, we have DERIVATION_NONE,
-  we'll wait for an explicit COLLATE clause which possibly can
-  come from another argument later: for example, this is valid,
-  but we don't know yet when collecting the first two arguments:
-     @code
-       CONCAT(latin1_swedish_ci_column,
-              latin1_german1_ci_column,
-              expr COLLATE latin1_german2_ci)
-  @endcode
-*/
-
-bool DTCollation::aggregate(DTCollation &dt, uint32_t flags)
-{
-  if (!my_charset_same(collation, dt.collation))
-  {
-    /* 
-       We do allow to use binary strings (like BLOBS)
-       together with character strings.
-       Binaries have more precedence than a character
-       string of the same derivation.
-    */
-    if (collation == &my_charset_bin)
-    {
-      if (derivation <= dt.derivation)
-	; // Do nothing
-      else
-      {
-	set(dt); 
-      }
-    }
-    else if (dt.collation == &my_charset_bin)
-    {
-      if (dt.derivation <= derivation)
-      {
-        set(dt);
-      }
-      else
-      {
-        // Do nothing
-      }
-    }
-    else if ((flags & MY_COLL_ALLOW_SUPERSET_CONV) &&
-             left_is_superset(this, &dt))
-    {
-      // Do nothing
-    }
-    else if ((flags & MY_COLL_ALLOW_SUPERSET_CONV) &&
-             left_is_superset(&dt, this))
-    {
-      set(dt);
-    }
-    else if ((flags & MY_COLL_ALLOW_COERCIBLE_CONV) &&
-             derivation < dt.derivation &&
-             dt.derivation >= DERIVATION_SYSCONST)
-    {
-      // Do nothing;
-    }
-    else if ((flags & MY_COLL_ALLOW_COERCIBLE_CONV) &&
-             dt.derivation < derivation &&
-             derivation >= DERIVATION_SYSCONST)
-    {
-      set(dt);
-    }
-    else
-    {
-      // Cannot apply conversion
-      set(0, DERIVATION_NONE, 0);
-      return 1;
-    }
-  }
-  else if (derivation < dt.derivation)
-  {
-    // Do nothing
-  }
-  else if (dt.derivation < derivation)
-  {
-    set(dt);
-  }
-  else
-  { 
-    if (collation == dt.collation)
-    {
-      // Do nothing
-    }
-    else 
-    {
-      if (derivation == DERIVATION_EXPLICIT)
-      {
-        set(0, DERIVATION_NONE, 0);
-        return 1;
-      }
-      if (collation->state & MY_CS_BINSORT)
-        return 0;
-      if (dt.collation->state & MY_CS_BINSORT)
-      {
-        set(dt);
-        return 0;
-      }
-      const CHARSET_INFO * const bin= get_charset_by_csname(collation->csname, 
-                                                            MY_CS_BINSORT,MYF(0));
-      set(bin, DERIVATION_NONE);
-    }
-  }
-  repertoire|= dt.repertoire;
-  return 0;
-}
-
-/******************************/
-static
-void my_coll_agg_error(DTCollation &c1, DTCollation &c2, const char *fname)
-{
-  my_error(ER_CANT_AGGREGATE_2COLLATIONS,MYF(0),
-           c1.collation->name,c1.derivation_name(),
-           c2.collation->name,c2.derivation_name(),
-           fname);
-}
-
-
-static
-void my_coll_agg_error(DTCollation &c1, DTCollation &c2, DTCollation &c3,
-                       const char *fname)
-{
-  my_error(ER_CANT_AGGREGATE_3COLLATIONS,MYF(0),
-  	   c1.collation->name,c1.derivation_name(),
-	   c2.collation->name,c2.derivation_name(),
-	   c3.collation->name,c3.derivation_name(),
-	   fname);
-}
-
-
-static
-void my_coll_agg_error(Item** args, uint32_t count, const char *fname,
-                       int item_sep)
-{
-  if (count == 2)
-    my_coll_agg_error(args[0]->collation, args[item_sep]->collation, fname);
-  else if (count == 3)
-    my_coll_agg_error(args[0]->collation, args[item_sep]->collation,
-                      args[2*item_sep]->collation, fname);
-  else
-    my_error(ER_CANT_AGGREGATE_NCOLLATIONS,MYF(0),fname);
-}
-
-
-bool agg_item_collations(DTCollation &c, const char *fname,
-                         Item **av, uint32_t count, uint32_t flags, int item_sep)
-{
-  uint32_t i;
-  Item **arg;
-  c.set(av[0]->collation);
-  for (i= 1, arg= &av[item_sep]; i < count; i++, arg++)
-  {
-    if (c.aggregate((*arg)->collation, flags))
-    {
-      my_coll_agg_error(av, count, fname, item_sep);
-      return true;
-    }
-  }
-  if ((flags & MY_COLL_DISALLOW_NONE) &&
-      c.derivation == DERIVATION_NONE)
-  {
-    my_coll_agg_error(av, count, fname, item_sep);
-    return true;
-  }
-  return false;
-}
-
-
-bool agg_item_collations_for_comparison(DTCollation &c, const char *fname,
-                                        Item **av, uint32_t count, uint32_t flags)
-{
-  return (agg_item_collations(c, fname, av, count,
-                              flags | MY_COLL_DISALLOW_NONE, 1));
-}
-
-
-/**
-  Collect arguments' character sets together.
-
-  We allow to apply automatic character set conversion in some cases.
-  The conditions when conversion is possible are:
-  - arguments A and B have different charsets
-  - A wins according to coercibility rules
-    (i.e. a column is stronger than a string constant,
-     an explicit COLLATE clause is stronger than a column)
-  - character set of A is either superset for character set of B,
-    or B is a string constant which can be converted into the
-    character set of A without data loss.
-    
-  If all of the above is true, then it's possible to convert
-  B into the character set of A, and then compare according
-  to the collation of A.
-  
-  For functions with more than two arguments:
-  @code
-    collect(A,B,C) ::= collect(collect(A,B),C)
-  @endcode
-  Since this function calls Session::change_item_tree() on the passed Item **
-  pointers, it is necessary to pass the original Item **'s, not copies.
-  Otherwise their values will not be properly restored (see BUG#20769).
-  If the items are not consecutive (eg. args[2] and args[5]), use the
-  item_sep argument, ie.
-  @code
-    agg_item_charsets(coll, fname, &args[2], 2, flags, 3)
-  @endcode
-*/
-
-bool agg_item_charsets(DTCollation &coll, const char *fname,
-                       Item **args, uint32_t nargs, uint32_t flags, int item_sep)
-{
-  Item **arg, *safe_args[2];
-
-  memset(safe_args, 0, sizeof(safe_args));
-
-  if (agg_item_collations(coll, fname, args, nargs, flags, item_sep))
-    return true;
-
-  /*
-    For better error reporting: save the first and the second argument.
-    We need this only if the the number of args is 3 or 2:
-    - for a longer argument list, "Illegal mix of collations"
-      doesn't display each argument's characteristics.
-    - if nargs is 1, then this error cannot happen.
-  */
-  if (nargs >=2 && nargs <= 3)
-  {
-    safe_args[0]= args[0];
-    safe_args[1]= args[item_sep];
-  }
-
-  Session *session= current_session;
-  bool res= false;
-  uint32_t i;
-
-  for (i= 0, arg= args; i < nargs; i++, arg+= item_sep)
-  {
-    Item* conv;
-    uint32_t dummy_offset;
-    if (!String::needs_conversion(0, (*arg)->collation.collation,
-                                  coll.collation,
-                                  &dummy_offset))
-      continue;
-
-    if (!(conv= (*arg)->safe_charset_converter(coll.collation)) &&
-        ((*arg)->collation.repertoire == MY_REPERTOIRE_ASCII))
-      conv= new Item_func_conv_charset(*arg, coll.collation, 1);
-
-    if (!conv)
-    {
-      if (nargs >=2 && nargs <= 3)
-      {
-        /* restore the original arguments for better error message */
-        args[0]= safe_args[0];
-        args[item_sep]= safe_args[1];
-      }
-      my_coll_agg_error(args, nargs, fname, item_sep);
-      res= true;
-      break; // we cannot return here, we need to restore "arena".
-    }
-    if ((*arg)->type() == Item::FIELD_ITEM)
-      ((Item_field *)(*arg))->no_const_subst= 1;
-    /*
-      If in statement prepare, then we create a converter for two
-      constant items, do it once and then reuse it.
-      If we're in execution of a prepared statement, arena is NULL,
-      and the conv was created in runtime memory. This can be
-      the case only if the argument is a parameter marker ('?'),
-      because for all true constants the charset converter has already
-      been created in prepare. In this case register the change for
-      rollback.
-    */
-    session->change_item_tree(arg, conv);
-    /*
-      We do not check conv->fixed, because Item_func_conv_charset which can
-      be return by safe_charset_converter can't be fixed at creation
-    */
-    conv->fix_fields(session, arg);
-  }
-
-  return res;
-}
-
-
 void Item_ident_for_show::make_field(Send_field *tmp_field)
 {
   tmp_field->table_name= tmp_field->org_table_name= table_name;

=== modified file 'drizzled/item.h'
--- drizzled/item.h	2008-11-12 17:42:40 +0000
+++ drizzled/item.h	2008-11-15 18:39:51 +0000
@@ -20,220 +20,13 @@
 #ifndef drizzled_item_h
 #define drizzled_item_h
 
+#include <drizzled/dtcollation.h>
+
 class Protocol;
 struct TableList;
 void item_init(void);			/* Init item functions */
 class Item_field;
 
-/*
-   "Declared Type Collation"
-   A combination of collation and its derivation.
-
-  Flags for collation aggregation modes:
-  MY_COLL_ALLOW_SUPERSET_CONV  - allow conversion to a superset
-  MY_COLL_ALLOW_COERCIBLE_CONV - allow conversion of a coercible value
-                                 (i.e. constant).
-  MY_COLL_ALLOW_CONV           - allow any kind of conversion
-                                 (combination of the above two)
-  MY_COLL_DISALLOW_NONE        - don't allow return DERIVATION_NONE
-                                 (e.g. when aggregating for comparison)
-  MY_COLL_CMP_CONV             - combination of MY_COLL_ALLOW_CONV
-                                 and MY_COLL_DISALLOW_NONE
-*/
-
-#define MY_COLL_ALLOW_SUPERSET_CONV   1
-#define MY_COLL_ALLOW_COERCIBLE_CONV  2
-#define MY_COLL_ALLOW_CONV            3
-#define MY_COLL_DISALLOW_NONE         4
-#define MY_COLL_CMP_CONV              7
-
-class DTCollation {
-public:
-  const CHARSET_INFO *collation;
-  enum Derivation derivation;
-  uint32_t repertoire;
-  
-  void set_repertoire_from_charset(const CHARSET_INFO * const cs)
-  {
-    repertoire= cs->state & MY_CS_PUREASCII ?
-                MY_REPERTOIRE_ASCII : MY_REPERTOIRE_UNICODE30;
-  }
-  DTCollation()
-  {
-    collation= &my_charset_bin;
-    derivation= DERIVATION_NONE;
-    repertoire= MY_REPERTOIRE_UNICODE30;
-  }
-  DTCollation(const CHARSET_INFO * const collation_arg, Derivation derivation_arg)
-  {
-    collation= collation_arg;
-    derivation= derivation_arg;
-    set_repertoire_from_charset(collation_arg);
-  }
-  void set(DTCollation &dt)
-  { 
-    collation= dt.collation;
-    derivation= dt.derivation;
-    repertoire= dt.repertoire;
-  }
-  void set(const CHARSET_INFO * const collation_arg, Derivation derivation_arg)
-  {
-    collation= collation_arg;
-    derivation= derivation_arg;
-    set_repertoire_from_charset(collation_arg);
-  }
-  void set(const CHARSET_INFO * const collation_arg,
-           Derivation derivation_arg,
-           uint32_t repertoire_arg)
-  {
-    collation= collation_arg;
-    derivation= derivation_arg;
-    repertoire= repertoire_arg;
-  }
-  void set(const CHARSET_INFO * const collation_arg)
-  {
-    collation= collation_arg;
-    set_repertoire_from_charset(collation_arg);
-  }
-  void set(Derivation derivation_arg)
-  { derivation= derivation_arg; }
-  bool aggregate(DTCollation &dt, uint32_t flags= 0);
-  bool set(DTCollation &dt1, DTCollation &dt2, uint32_t flags= 0)
-  { set(dt1); return aggregate(dt2, flags); }
-  const char *derivation_name() const
-  {
-    switch(derivation)
-    {
-      case DERIVATION_IGNORABLE: return "IGNORABLE";
-      case DERIVATION_COERCIBLE: return "COERCIBLE";
-      case DERIVATION_IMPLICIT:  return "IMPLICIT";
-      case DERIVATION_SYSCONST:  return "SYSCONST";
-      case DERIVATION_EXPLICIT:  return "EXPLICIT";
-      case DERIVATION_NONE:      return "NONE";
-      default: return "UNKNOWN";
-    }
-  }
-};
-
-/*************************************************************************/
-/*
-  A framework to easily handle different return types for hybrid items
-  (hybrid item is an item whose operand can be of any type, e.g. integer,
-  real, decimal).
-*/
-
-struct Hybrid_type_traits;
-
-struct Hybrid_type
-{
-  int64_t integer;
-
-  double real;
-  /*
-    Use two decimal buffers interchangeably to speed up += operation
-    which has no native support in decimal library.
-    Hybrid_type+= arg is implemented as dec_buf[1]= dec_buf[0] + arg.
-    The third decimal is used as a handy temporary storage.
-  */
-  my_decimal dec_buf[3];
-  int used_dec_buf_no;
-
-  /*
-    Traits moved to a separate class to
-      a) be able to easily change object traits in runtime
-      b) they work as a differentiator for the union above
-  */
-  const Hybrid_type_traits *traits;
-
-  Hybrid_type() {}
-  /* XXX: add traits->copy() when needed */
-  Hybrid_type(const Hybrid_type &rhs) :traits(rhs.traits) {}
-};
-
-
-/* Hybryd_type_traits interface + default implementation for REAL_RESULT */
-
-struct Hybrid_type_traits
-{
-  virtual Item_result type() const { return REAL_RESULT; }
-
-  virtual void
-  fix_length_and_dec(Item *item, Item *arg) const;
-
-  /* Hybrid_type operations. */
-  virtual void set_zero(Hybrid_type *val) const { val->real= 0.0; }
-  virtual void add(Hybrid_type *val, Field *f) const
-  { val->real+= f->val_real(); }
-  virtual void div(Hybrid_type *val, uint64_t u) const
-  { val->real/= uint64_t2double(u); }
-
-  virtual int64_t val_int(Hybrid_type *val,
-                          bool unsigned_flag) const;
-  virtual double val_real(Hybrid_type *val) const { return val->real; }
-  virtual my_decimal *val_decimal(Hybrid_type *val, my_decimal *buf) const;
-  virtual String *val_str(Hybrid_type *val, String *buf, uint8_t decimals) const;
-  static const Hybrid_type_traits *instance();
-  Hybrid_type_traits() {}
-  virtual ~Hybrid_type_traits() {}
-};
-
-
-struct Hybrid_type_traits_decimal: public Hybrid_type_traits
-{
-  virtual Item_result type() const { return DECIMAL_RESULT; }
-
-  virtual void
-  fix_length_and_dec(Item *arg, Item *item) const;
-
-  /* Hybrid_type operations. */
-  virtual void set_zero(Hybrid_type *val) const;
-  virtual void add(Hybrid_type *val, Field *f) const;
-  virtual void div(Hybrid_type *val, uint64_t u) const;
-
-  virtual int64_t val_int(Hybrid_type *val, bool unsigned_flag) const;
-  virtual double val_real(Hybrid_type *val) const;
-  virtual my_decimal *val_decimal(Hybrid_type *val,
-                                  my_decimal *buf __attribute__((unused))) const
-  { return &val->dec_buf[val->used_dec_buf_no]; }
-  virtual String *val_str(Hybrid_type *val, String *buf, uint8_t decimals) const;
-  static const Hybrid_type_traits_decimal *instance();
-  Hybrid_type_traits_decimal() {};
-};
-
-
-struct Hybrid_type_traits_integer: public Hybrid_type_traits
-{
-  virtual Item_result type() const { return INT_RESULT; }
-
-  virtual void
-  fix_length_and_dec(Item *arg, Item *item) const;
-
-  /* Hybrid_type operations. */
-  virtual void set_zero(Hybrid_type *val) const
-  { val->integer= 0; }
-  virtual void add(Hybrid_type *val, Field *f) const
-  { val->integer+= f->val_int(); }
-  virtual void div(Hybrid_type *val, uint64_t u) const
-  { val->integer/= (int64_t) u; }
-
-  virtual int64_t val_int(Hybrid_type *val,
-                           bool unsigned_flag __attribute__((unused))) const
-  { return val->integer; }
-  virtual double val_real(Hybrid_type *val) const
-  { return (double) val->integer; }
-  virtual my_decimal *val_decimal(Hybrid_type *val,
-                                  my_decimal *buf __attribute__((unused))) const
-  {
-    int2my_decimal(E_DEC_FATAL_ERROR, val->integer, 0, &val->dec_buf[2]);
-    return &val->dec_buf[2];
-  }
-  virtual String *val_str(Hybrid_type *val, String *buf,
-                          uint8_t decimals __attribute__((unused))) const
-  { buf->set(val->integer, &my_charset_bin); return buf;}
-  static const Hybrid_type_traits_integer *instance();
-  Hybrid_type_traits_integer() {};
-};
-
 
 void dummy_error_processor(Session *session, void *data);
 
@@ -384,43 +177,14 @@
 };
 
 
-/*
-  This enum is used to report information about monotonicity of function
-  represented by Item* tree.
-  Monotonicity is defined only for Item* trees that represent table
-  partitioning expressions (i.e. have no subselects/user vars/PS parameters
-  etc etc). An Item* tree is assumed to have the same monotonicity properties
-  as its correspoinding function F:
-
-  [signed] int64_t F(field1, field2, ...) {
-    put values of field_i into table record buffer;
-    return item->val_int(); 
-  }
-
-  NOTE
-  At the moment function monotonicity is not well defined (and so may be
-  incorrect) for Item trees with parameters/return types that are different
-  from INT_RESULT, may be NULL, or are unsigned.
-  It will be possible to address this issue once the related partitioning bugs
-  (BUG#16002, BUG#15447, BUG#13436) are fixed.
-*/
-
-typedef enum monotonicity_info 
-{
-   NON_MONOTONIC,              /* none of the below holds */
-   MONOTONIC_INCREASING,       /* F() is unary and (x < y) => (F(x) <= F(y)) */
-   MONOTONIC_STRICT_INCREASING /* F() is unary and (x < y) => (F(x) <  F(y)) */
-} enum_monotonicity_info;
-
 /*************************************************************************/
-typedef bool (Item::*Item_processor) (unsigned char *arg);
 /*
   Analyzer function
     SYNOPSIS
       argp   in/out IN:  Analysis parameter
                     OUT: Parameter to be passed to the transformer
 
-    RETURN 
+     RETURN
       true   Invoke the transformer
       false  Don't do it
 
@@ -428,6 +192,7 @@
 typedef bool (Item::*Item_analyzer) (unsigned char **argp);
 typedef Item* (Item::*Item_transformer) (unsigned char *arg);
 typedef void (*Cond_traverser) (const Item *item, void *arg);
+typedef bool (Item::*Item_processor) (unsigned char *arg);
 
 
 class Item
@@ -1002,13 +767,6 @@
   }
 };
 
-bool agg_item_collations(DTCollation &c, const char *name,
-                         Item **items, uint32_t nitems, uint32_t flags, int item_sep);
-bool agg_item_collations_for_comparison(DTCollation &c, const char *name,
-                                        Item **items, uint32_t nitems, uint32_t flags);
-bool agg_item_charsets(DTCollation &c, const char *name,
-                       Item **items, uint32_t nitems, uint32_t flags, int item_sep);
-
 
 class Item_num: public Item_basic_constant
 {

=== modified file 'drizzled/item_cmpfunc.cc'
--- drizzled/item_cmpfunc.cc	2008-11-05 05:50:56 +0000
+++ drizzled/item_cmpfunc.cc	2008-11-16 02:30:08 +0000
@@ -219,15 +219,6 @@
   return found_types;
 }
 
-static void my_coll_agg_error(DTCollation &c1, DTCollation &c2,
-                              const char *fname)
-{
-  my_error(ER_CANT_AGGREGATE_2COLLATIONS, MYF(0),
-           c1.collation->name,c1.derivation_name(),
-           c2.collation->name,c2.derivation_name(),
-           fname);
-}
-
 
 Item_bool_func2* Eq_creator::create(Item *a, Item *b) const
 {

=== modified file 'drizzled/item_sum.cc'
--- drizzled/item_sum.cc	2008-11-05 05:50:56 +0000
+++ drizzled/item_sum.cc	2008-11-16 02:30:08 +0000
@@ -23,6 +23,11 @@
 #include <drizzled/server_includes.h>
 #include <drizzled/sql_select.h>
 #include <drizzled/error.h>
+#include <drizzled/hybrid_type_traits.h>
+#include <drizzled/hybrid_type_traits_integer.h>
+#include <drizzled/hybrid_type_traits_decimal.h>
+
+
 #include CMATH_H
 
 #if defined(CMATH_NAMESPACE)
@@ -939,6 +944,7 @@
   return &fast_decimal_traits_instance;
 }
 
+
 void Item_sum_distinct::fix_length_and_dec()
 {
   assert(args[0]->fixed);
@@ -980,6 +986,12 @@
 }
 
 
+enum Item_result Item_sum_distinct::result_type () const
+{
+  return val.traits->type();
+}
+
+
 /**
   @todo
   check that the case of CHAR(0) works OK

=== modified file 'drizzled/item_sum.h'
--- drizzled/item_sum.h	2008-11-07 00:15:51 +0000
+++ drizzled/item_sum.h	2008-11-15 18:39:51 +0000
@@ -24,6 +24,7 @@
 
 
 #include <mysys/my_tree.h>
+#include <drizzled/hybrid_type.h>
 
 /*
   Class Item_sum is the base class used for special expressions that SQL calls
@@ -488,7 +489,7 @@
   void update_field() {} // not used
   virtual void no_rows_in_result() {}
   void fix_length_and_dec();
-  enum Item_result result_type () const { return val.traits->type(); }
+  enum Item_result result_type () const;
   virtual void calculate_val_and_count();
   virtual bool unique_walk_function(void *elem);
 };

